//////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file RTTC/ForceFeedback/BasicImpl/StateChangeCallbackFactoryImpl.h
 * \version 5.2.1
 *
 * \brief Contains the declaration of and documentation for
 * RTTC::ForceFeedback::BasicImpl::StateChangeCallbackFactoryImpl.
 *
 * This file contains the declaration of and documentation for the
 * RTTC::ForceFeedback::BasicImpl::StateChangeCallbackFactoryImpl class.
 */

#ifndef RTTC_ForceFeedback_BasicImpl_StateChangeCallbackFactoryImpl_h
#define RTTC_ForceFeedback_BasicImpl_StateChangeCallbackFactoryImpl_h

#include <DCT/System/config.h>
#include <basicImpl_RTTC_TASemulator_v4_EXPORT.h>
#include <RTTC/ForceFeedback/StateChangeCallbackFactory.h>

#include "StateChangeCallbackImpl.h"
#include "CallbackInfo.h"

// The example applications provided with the TENA Middleware utilize namespaces
// to separate class declarations and definitions.  This is not required, but is
// a recommended C++ practice.
namespace RTTC
{
  namespace ForceFeedback
  {
    namespace BasicImpl
    {
      /*!\class StateChangeCallbackFactoryImpl StateChangeCallbackFactoryImpl.h \
         RTTC/ForceFeedback/BasicImpl/StateChangeCallbackFactoryImpl.h
       *
       * \brief A Basic concrete implementation of an
       * RTTC::ForceFeedback::StateChangeCallbackFactory.
       *
       * This basic implementation uses the CallbackInfo to report state changes to an output
       * stream
       */
      class basicImpl_RTTC_TASemulator_v4_EXPORT StateChangeCallbackFactoryImpl
        : public virtual RTTC::ForceFeedback::StateChangeCallbackFactory
      {
      public:

        /*!
          The basic implementation constructor takes a single argument -- the CallbackInfo
          with a list of discovered RTTC::ForceFeedback-s, (used by discovery and
          destruction callbacks), and a std::ostream used by the stateChange callback.
        */
        StateChangeCallbackFactoryImpl( CallbackInfoPtr const & );

        /*!
          The basic implementation destructor has no behavior.  It is conceivable that
          some applications require behavior here, but most will probably not.
        */
        ~StateChangeCallbackFactoryImpl();

        //! Create a Callback and return it to the TENA Middleware.
        /*!
          \param[in] pProxy A ProxyPtr for the new RTTC::ForceFeedback::Servant that
          matches a current subscription has been detected by the TENA Middleware.

          \return An object that implements the TENA::Middleware::Callback::Callback
          interface.  Ownership of the memory is passed to the invoker.

          This method is invoked by the TENA Middleware when a new instance of an
          RTTC::ForceFeedback::Servant that matches a subscription request is
          detected.  The returned Callback is placed on an internal queue, with
          callbacks removed from the queue and executed in response to user calls to
          session::evokeCallback()
        */
        std::auto_ptr< TENA::Middleware::Callback::Callback >
        create( RTTC::ForceFeedback::ProxyPtr const & pProxy );

      private:
        //! Hold a CallbackInfo to communicate between main pgm and callbacks
        CallbackInfoPtr _pCallbackInfo;

        //! Default copy construction is disallowed.
        StateChangeCallbackFactoryImpl(
          StateChangeCallbackFactoryImpl const & rhs );
        //! Default copy assignment is disallowed.
        StateChangeCallbackFactoryImpl & operator=(
          StateChangeCallbackFactoryImpl const & rhs );

      };
    } // End of namespace BasicImpl
  } // End of namespace ForceFeedback
} // End of namespace RTTC


#endif // ifndef RTTC_ForceFeedback_BasicImpl_StateChangeCallbackFactoryImpl_h
