/////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file RTTC/ForceFeedback/BasicImpl/StateChangeCallbackImpl.cpp
 * \version 5.2.1
 *
 * \brief Contains the definition of methods of
 * RTTC::ForceFeedback::BasicImpl::StateChangeCallbackImpl.
 *
 * This file contains the definition of the methods of the
 * RTTC::ForceFeedback::BasicImpl::StateChangeCallbackImpl class.
 */

#include "StateChangeCallbackImpl.h"


// For SDO and any contained SDO types, include headers to allow access to
// SDO publication state.
#include <RTTC/ForceFeedback/ImmutablePublicationState.h>
#include <RTTC/ForceFeedback/Proxy.h>


#include <string>
#include <exception>
#include <iostream>
#include <iomanip>

///////////////////////////////////////////////////////////////////////////////
RTTC::ForceFeedback::BasicImpl::
StateChangeCallbackImpl::
~StateChangeCallbackImpl()
{
  // Basic Implementation has no required behavior in state change callback
  // destructor.  Might need to free resources, close database connections,
  // etc.
}

///////////////////////////////////////////////////////////////////////////////
RTTC::ForceFeedback::BasicImpl::
StateChangeCallbackImpl::
StateChangeCallbackImpl(
  RTTC::ForceFeedback::ProxyPtr const & pProxy,
  CallbackInfoPtr const & pCallbackInfo
)
  :
  _pCallbackInfo( pCallbackInfo ),
  _pProxy( pProxy )
{
  // Basic StateChangeCallback implementation requires no behavior at callback
  // creation.  This is fairly typical--most callback constructors will not
  // have behavior.
}

///////////////////////////////////////////////////////////////////////////////
void
RTTC::ForceFeedback::BasicImpl::
StateChangeCallbackImpl::
execute()
{
  if ( 1 < _pCallbackInfo->getVerbosity() )
  {
    // This is just an example that performs some "update" every time the state
    // changes; in this case, printing out the new SDO state.
    std::ostream & os = _pCallbackInfo->getOutputStream();

    os << "RTTC::ForceFeedback::BasicImpl StateChangeCallbackImpl execute()"
       << std::endl;
    os << "SDO ID " << _pProxy->getSDOid() << std::endl;
    os << "Version "
       << _pProxy->getPublicationState()->get_RTTC_ForceFeedback_VersionCounter()
       << std::endl;
  }
  else if ( 1 == _pCallbackInfo->getVerbosity() )
  {
    std::ostream & os = _pCallbackInfo->getOutputStream();
    os << 'c' << std::flush;
  }

  if ( 2 < _pCallbackInfo->getVerbosity() )
  {
    std::ostream & os = _pCallbackInfo->getOutputStream();
    //
    // Get the publicationState of the
    // RTTC::ForceFeedback SDO '_pProxy'.
    //
    RTTC::ForceFeedback::ImmutablePublicationStatePtr
      this_ForceFeedback_PubState( _pProxy->getPublicationState() );
    os << std::setw(0) << " " << "this.RTTC_ForceFeedback_VersionCounter "
              << this_ForceFeedback_PubState->get_RTTC_ForceFeedback_VersionCounter() << std::endl;
    os << std::setw(0) << " " << "this.magnitude ";
    os << this_ForceFeedback_PubState->get_magnitude() << std::endl;
    os << std::setw(0) << " " << "this.period ";
    os << this_ForceFeedback_PubState->get_period() << std::endl;
    os << std::setw(0) << " " << "this.isActive ";
    os << (this_ForceFeedback_PubState->get_isActive() ? "true":"false") << std::endl;
  }

  // Callbacks should only catch exceptions if they can actually HANDLE the
  // exception--i.e. correct the error condition.  Otherwise, the exception
  // will propagate up to the evokeCallbacks invocation where it may be handled
  // in an application specific manner.
}
