//////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file RTTC/SceneGenerator/BasicImpl/DiscoveryCallbackFactoryImpl.h
 * \version 5.2.1
 *
 * \brief Contains the declaration of and documentation for
 * RTTC::SceneGenerator::BasicImpl::DiscoveryCallbackFactoryImpl.
 *
 * This file contains the declaration of and documentation for the
 * RTTC::SceneGenerator::BasicImpl::DiscoveryCallbackFactoryImpl class.
 */

#ifndef RTTC_SceneGenerator_BasicImpl_DiscoveryCallbackFactoryImpl_h
#define RTTC_SceneGenerator_BasicImpl_DiscoveryCallbackFactoryImpl_h

#include <DCT/System/config.h>
#include <basicImpl_RTTC_TASemulator_v4_EXPORT.h>
#include <RTTC/SceneGenerator/DiscoveryCallbackFactory.h>

#include "DiscoveryCallbackImpl.h"
#include "CallbackInfo.h"

namespace RTTC
{
  namespace SceneGenerator
  {
    namespace BasicImpl
    {
      /*!\class DiscoveryCallbackFactoryImpl DiscoveryCallbackFactoryImpl.h \
         RTTC/SceneGenerator/BasicImpl/DiscoveryCallbackFactoryImpl.h
       *
       * \brief A Basic concrete implementation of an
       * RTTC::SceneGenerator::DiscoveryCallbackFactory.
       *
       * This basic implementation uses the CallbackInfo to add newly discovered SDOs to
       * the list of discovered SDOs
       */
      class basicImpl_RTTC_TASemulator_v4_EXPORT DiscoveryCallbackFactoryImpl
        : public virtual RTTC::SceneGenerator::DiscoveryCallbackFactory
      {
      public:

        /*!
          The basic implementation constructor takes a single argument -- the CallbackInfo
          with a list of discovered RTTC::SceneGenerator-s, (used by discovery and
          destruction callbacks), and a std::ostream used by the stateChange callback.
        */
        DiscoveryCallbackFactoryImpl( CallbackInfoPtr const & );

        /*!
          The basic implementation destructor has no behavior.  It is conceivable that
          some applications require behavior here, but most will probably not.
        */
        ~DiscoveryCallbackFactoryImpl();

        //! Create a Callback and return it to the TENA Middleware.
        /*!
          \param[in] pProxy A ProxyPtr for the new RTTC::SceneGenerator::Servant that
          matches a current subscription has been detected by the TENA Middleware.

          \return An object that implements the TENA::Middleware::Callback::Callback
          interface.  Ownership of the memory is passed to the invoker.

          This method is invoked by the TENA Middleware when a new instance of an
          RTTC::SceneGenerator::Servant that matches a subscription request is
          detected.  The returned Callback is placed on an internal queue, with
          callbacks removed from the queue and executed in response to user calls to
          session::evokeCallback()
        */
        std::auto_ptr< TENA::Middleware::Callback::Callback >
        create( RTTC::SceneGenerator::ProxyPtr const & pProxy );

      private:
        //! Hold a CallbackInfo to communicate between main pgm and callbacks
        CallbackInfoPtr _pCallbackInfo;

        //! Default copy construction is disallowed.
        DiscoveryCallbackFactoryImpl(
          DiscoveryCallbackFactoryImpl const & rhs );
        //! Default copy assignment is disallowed.
        DiscoveryCallbackFactoryImpl & operator=(
          DiscoveryCallbackFactoryImpl const & rhs );
      };
#ifndef DOXYGEN_SHOULD_SKIP_THIS
      // Typedef to prevent eclipsing of this class by function of the same name
      typedef DiscoveryCallbackFactoryImpl DiscoveryCallbackFactoryImpl;
#endif // DOXYGEN_SHOULD_SKIP_THIS
    } // End of namespace BasicImpl
  } // End of namespace SceneGenerator
} // End of namespace RTTC

#endif // ifndef RTTC_SceneGenerator_BasicImpl_DiscoveryCallbackFactoryImpl_h
