/////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file RTTC/SceneGenerator/BasicImpl/StateChangeCallbackImpl.cpp
 * \version 5.2.1
 *
 * \brief Contains the definition of methods of
 * RTTC::SceneGenerator::BasicImpl::StateChangeCallbackImpl.
 *
 * This file contains the definition of the methods of the
 * RTTC::SceneGenerator::BasicImpl::StateChangeCallbackImpl class.
 */

#include "StateChangeCallbackImpl.h"


// For SDO and any contained SDO types, include headers to allow access to
// SDO publication state.
#include <RTTC/SceneGenerator/ImmutablePublicationState.h>
#include <RTTC/SceneGenerator/Proxy.h>

// For any localclass or message types, include the header and factory
// *implementation* needed to create and access the local class
#include <TENA/UniqueID/Includes.h>


#include <string>
#include <exception>
#include <iostream>
#include <iomanip>

///////////////////////////////////////////////////////////////////////////////
RTTC::SceneGenerator::BasicImpl::
StateChangeCallbackImpl::
~StateChangeCallbackImpl()
{
  // Basic Implementation has no required behavior in state change callback
  // destructor.  Might need to free resources, close database connections,
  // etc.
}

///////////////////////////////////////////////////////////////////////////////
RTTC::SceneGenerator::BasicImpl::
StateChangeCallbackImpl::
StateChangeCallbackImpl(
  RTTC::SceneGenerator::ProxyPtr const & pProxy,
  CallbackInfoPtr const & pCallbackInfo
)
  :
  _pCallbackInfo( pCallbackInfo ),
  _pProxy( pProxy )
{
  // Basic StateChangeCallback implementation requires no behavior at callback
  // creation.  This is fairly typical--most callback constructors will not
  // have behavior.
}

///////////////////////////////////////////////////////////////////////////////
void
RTTC::SceneGenerator::BasicImpl::
StateChangeCallbackImpl::
execute()
{
  if ( 1 < _pCallbackInfo->getVerbosity() )
  {
    // This is just an example that performs some "update" every time the state
    // changes; in this case, printing out the new SDO state.
    std::ostream & os = _pCallbackInfo->getOutputStream();

    os << "RTTC::SceneGenerator::BasicImpl StateChangeCallbackImpl execute()"
       << std::endl;
    os << "SDO ID " << _pProxy->getSDOid() << std::endl;
    os << "Version "
       << _pProxy->getPublicationState()->get_RTTC_SceneGenerator_VersionCounter()
       << std::endl;
  }
  else if ( 1 == _pCallbackInfo->getVerbosity() )
  {
    std::ostream & os = _pCallbackInfo->getOutputStream();
    os << 'c' << std::flush;
  }

  if ( 2 < _pCallbackInfo->getVerbosity() )
  {
    std::ostream & os = _pCallbackInfo->getOutputStream();
    //
    // Get the publicationState of the
    // RTTC::SceneGenerator SDO '_pProxy'.
    //
    RTTC::SceneGenerator::ImmutablePublicationStatePtr
      this_SceneGenerator_PubState( _pProxy->getPublicationState() );
    os << std::setw(0) << " " << "this.RTTC_SceneGenerator_VersionCounter "
              << this_SceneGenerator_PubState->get_RTTC_SceneGenerator_VersionCounter() << std::endl;
    os << std::setw(0) << " " << "this.sceneGeneratorID ";
    TENA::UniqueID::Pointer sceneGeneratorID_LocalClass_Ptr(
      this_SceneGenerator_PubState->get_sceneGeneratorID() );

    os << std::endl;
    os << std::setw(4) << " " << "sceneGeneratorID.siteID ";
    os << sceneGeneratorID_LocalClass_Ptr->get_siteID() << std::endl;
    os << std::setw(4) << " " << "sceneGeneratorID.applicationID ";
    os << sceneGeneratorID_LocalClass_Ptr->get_applicationID() << std::endl;
    os << std::setw(4) << " " << "sceneGeneratorID.objectID ";
    os << sceneGeneratorID_LocalClass_Ptr->get_objectID() << std::endl;
    os << std::setw(0) << " " << "this.endPoint ";
    os << this_SceneGenerator_PubState->get_endPoint() << std::endl;
    os << std::setw(0) << " " << "this.terrainDescription ";
    os << this_SceneGenerator_PubState->get_terrainDescription() << std::endl;
  }

  // Callbacks should only catch exceptions if they can actually HANDLE the
  // exception--i.e. correct the error condition.  Otherwise, the exception
  // will propagate up to the evokeCallbacks invocation where it may be handled
  // in an application specific manner.
}
