/////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file RTTC/TASemulator/BasicImpl/SubscriptionInfoImpl.h
 * \version 5.2.1
 *
 * \brief Contains the declaration of and documentation for
 * RTTC::TASemulator::BasicImpl::SubscriptionInfoImpl.
 *
 * This file contains the declaration of and documentation for the
 * RTTC::TASemulator::BasicImpl::SubscriptionInfoImpl class.
 */

#ifndef RTTC_TASemulator_BasicImpl_SubscriptionInfoImpl_h
#define RTTC_TASemulator_BasicImpl_SubscriptionInfoImpl_h

#include <DCT/System/config.h>
#include <basicImpl_RTTC_TASemulator_v4_EXPORT.h>
#include "CallbackInfo.h"
#include <RTTC/TASemulator/SubscriptionInfo.h>

namespace RTTC
{
  namespace TASemulator
  {
    namespace BasicImpl
    {
      /*!\class SubscriptionInfoImpl SubscriptionInfoImpl.h \
         RTTC/TASemulator/BasicImpl/SubscriptionInfoImpl.h
       *
       * \brief Holder for all user-defined objects required to subscribe
       * to this SDO type.
       *
       * The basic implementation for this object should be sufficient
       * for most applications.
       *
       * Any custom variables needed by the callback factories can be
       * passed in through the CallbackInfo object.
       */
      class basicImpl_RTTC_TASemulator_v4_EXPORT SubscriptionInfoImpl
        : public RTTC::TASemulator::SubscriptionInfo
      {
      public:
        // If no callbacks are desired, construct a default SubscriptionInfo
        // from the OM definition.  This is typically used to dereference an
        // SDO pointer.

        /*!
         * This ctor creates callback factories using the provided
         * CallbackInfoPtr .
         *
         * Use this ctor when subscribing to this SDO type.
         */
        explicit SubscriptionInfoImpl( CallbackInfoPtr const & );

        virtual
        ~SubscriptionInfoImpl();

      private:
        //! Default copy construction is disallowed.
        SubscriptionInfoImpl(
          SubscriptionInfoImpl const & rhs );
        //! Default copy assignment is disallowed.
        SubscriptionInfoImpl & operator=(
          SubscriptionInfoImpl const & rhs );

      };
    } // End of namespace BasicImpl
  } // End of namespace TASemulator
} // End of namespace RTTC

#endif // ifndef RTTC_TASemulator_BasicImpl_SubscriptionInfoImpl_h
