//////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file TENA/Platform/BasicImpl/DestructionCallbackImpl.h
 * \version 5.2.1
 *
 * \brief Contains the declaration of and documentation for
 * TENA::Platform::BasicImpl::DestructionCallbackImpl.
 *
 * This file contains the declaration of and documentation for the
 * TENA::Platform::BasicImpl::DestructionCallbackImpl class.
 */

#ifndef TENA_Platform_BasicImpl_DestructionCallbackImpl_h
#define TENA_Platform_BasicImpl_DestructionCallbackImpl_h

#include <DCT/System/config.h>
#include <basicImpl_TENA_Platform_v3_1_EXPORT.h>
#include <TENA/Middleware/Callback/Callback.h>
#include <TENA/Platform/Proxy.h>

#include "CallbackInfo.h"

namespace TENA
{
  namespace Platform
  {
    namespace BasicImpl
    {
      /*!\class DestructionCallbackImpl DestructionCallbackImpl.h \
         TENA/Platform/BasicImpl/DestructionCallbackImpl.h
       *
       * \brief Callback created in response to destruction of an SDO
       *
       * When an existing SDO is destroyed, a DestructionCallback is created.  This
       * basic simply removes the destroyed SDO from a list of SDOs.  To alter the behavior,
       * modify the execute method of the callback.
       */
      class basicImpl_TENA_Platform_v3_1_EXPORT DestructionCallbackImpl
        : public virtual TENA::Middleware::Callback::Callback
      {
      public:
        //! This is the only constructor.
        /*!
          \param[in] pProxy The ProxyPtr for the Servant being destroyed.
          \param[in] pCallbackInfo The CallbackInfo object containing the list of ProxyPtrs that have been discovered.

          This basic implementation simply removes the destroyed SDO ProxyPtr
          from the list of discovered SDOs in the CallbackInfo
        */
        DestructionCallbackImpl(
          TENA::Platform::ProxyPtr const & pProxy,
          CallbackInfoPtr const & pCallbackInfo );

        //! Instances are destructed after invoking the execute() method.
        ~DestructionCallbackImpl();

        //! Perform the behavior associated with this callback.
        /*!
         * Basic implementation behavior just removes the destroyed SDO ProxyPtr
         * from the std::list in the CallbackInfo
         */
        void
        execute();

      private:
        //! Default copy construction is disallowed.
        DestructionCallbackImpl(
          DestructionCallbackImpl const & rhs );
        //! Default copy assignment is disallowed.
        DestructionCallbackImpl & operator=(
          DestructionCallbackImpl const & rhs );

        //! Hold a CallbackInfo
        CallbackInfoPtr  _pCallbackInfo;

        //! Hold a Proxy
        TENA::Platform::ProxyPtr _pProxy;
      };
#ifndef DOXYGEN_SHOULD_SKIP_THIS
      // Typedef to prevent eclipsing of this class by function of the same name
      typedef DestructionCallbackImpl DestructionCallbackImpl;
#endif // DOXYGEN_SHOULD_SKIP_THIS
    } // End of namespace BasicImpl
  } // End of namespace Platform
} // End of namespace TENA

#endif // ifndef TENA_Platform_BasicImpl_DestructionCallbackImpl_h
