/////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file TENA/PlatformExtension/BasicImpl/StateChangeCallbackImpl.cpp
 * \version 5.2.1
 *
 * \brief Contains the definition of methods of
 * TENA::PlatformExtension::BasicImpl::StateChangeCallbackImpl.
 *
 * This file contains the definition of the methods of the
 * TENA::PlatformExtension::BasicImpl::StateChangeCallbackImpl class.
 */

#include "StateChangeCallbackImpl.h"


// For SDO and any contained SDO types, include headers to allow access to
// SDO publication state.
#include <TENA/PlatformExtension/ImmutablePublicationState.h>
#include <TENA/PlatformExtension/Proxy.h>

// For any localclass or message types, include the header and factory
// *implementation* needed to create and access the local class
#include <TENA/UniqueID/Includes.h>
#include <TENA/Time/Includes.h>
#include <TENA/CoordinatedUniversalTime/Includes.h>
#include <TENA/Date/Includes.h>
#include <TENA/GpsTime/Includes.h>
#include <TENA/UnixTime/Includes.h>


#include <string>
#include <exception>
#include <iostream>
#include <iomanip>

///////////////////////////////////////////////////////////////////////////////
TENA::PlatformExtension::BasicImpl::
StateChangeCallbackImpl::
~StateChangeCallbackImpl()
{
  // Basic Implementation has no required behavior in state change callback
  // destructor.  Might need to free resources, close database connections,
  // etc.
}

///////////////////////////////////////////////////////////////////////////////
TENA::PlatformExtension::BasicImpl::
StateChangeCallbackImpl::
StateChangeCallbackImpl(
  TENA::PlatformExtension::ProxyPtr const & pProxy,
  CallbackInfoPtr const & pCallbackInfo
)
  :
  _pCallbackInfo( pCallbackInfo ),
  _pProxy( pProxy )
{
  // Basic StateChangeCallback implementation requires no behavior at callback
  // creation.  This is fairly typical--most callback constructors will not
  // have behavior.
}

///////////////////////////////////////////////////////////////////////////////
void
TENA::PlatformExtension::BasicImpl::
StateChangeCallbackImpl::
execute()
{
  if ( 1 < _pCallbackInfo->getVerbosity() )
  {
    // This is just an example that performs some "update" every time the state
    // changes; in this case, printing out the new SDO state.
    std::ostream & os = _pCallbackInfo->getOutputStream();

    os << "TENA::PlatformExtension::BasicImpl StateChangeCallbackImpl execute()"
       << std::endl;
    os << "SDO ID " << _pProxy->getSDOid() << std::endl;
    os << "Version "
       << _pProxy->getPublicationState()->get_TENA_PlatformExtension_VersionCounter()
       << std::endl;
  }
  else if ( 1 == _pCallbackInfo->getVerbosity() )
  {
    std::ostream & os = _pCallbackInfo->getOutputStream();
    os << 'c' << std::flush;
  }

  if ( 2 < _pCallbackInfo->getVerbosity() )
  {
    std::ostream & os = _pCallbackInfo->getOutputStream();
    //
    // Get the publicationState of the
    // TENA::PlatformExtension SDO '_pProxy'.
    //
    TENA::PlatformExtension::ImmutablePublicationStatePtr
      this_PlatformExtension_PubState( _pProxy->getPublicationState() );
    os << std::setw(0) << " " << "this.TENA_PlatformExtension_VersionCounter "
              << this_PlatformExtension_PubState->get_TENA_PlatformExtension_VersionCounter() << std::endl;
    os << std::setw(0) << " " << "this.platformID ";
    TENA::UniqueID::Pointer platformID_LocalClass_Ptr(
      this_PlatformExtension_PubState->get_platformID() );

    os << std::endl;
    os << std::setw(4) << " " << "platformID.siteID ";
    os << platformID_LocalClass_Ptr->get_siteID() << std::endl;
    os << std::setw(4) << " " << "platformID.applicationID ";
    os << platformID_LocalClass_Ptr->get_applicationID() << std::endl;
    os << std::setw(4) << " " << "platformID.objectID ";
    os << platformID_LocalClass_Ptr->get_objectID() << std::endl;
    os << std::setw(0) << " " << "this.timeStamp ";
    TENA::Time::Pointer timeStamp_LocalClass_Ptr(
      this_PlatformExtension_PubState->get_timeStamp() );

    os << std::endl;
    os << std::setw(0) << " " << "this.extensionTypeID ";
    os << this_PlatformExtension_PubState->get_extensionTypeID() << std::endl;
  }

  // Callbacks should only catch exceptions if they can actually HANDLE the
  // exception--i.e. correct the error condition.  Otherwise, the exception
  // will propagate up to the evokeCallbacks invocation where it may be handled
  // in an application specific manner.
}
