//  Target Organization:  RTTC Dynamic Test Branch (TA-2)
//  Develpment Organization : Trideum Corporation
//  Developer : Mike Greene   mgreene@trideum.com
//  Developer's Phone :  (256) 704-6113
//  OpenSceneGraph Version 2.78  (www.openscenegraph.org)
//
//  Last Mod 04-09-2009
//
#include "stdafx.h"

// TENA defines and includes

#include <TENA/Middleware/Config.h>

// Open Scene Graph includes
#include <osg/Geometry>

#include <osgDB/ReadFile>
#include <osgDB/FileUtils>
#include <osg/Node>

#include <osg/MatrixTransform>
#include <osg/PositionAttitudeTransform>

#include <osgDB/ReadFile>
#include <osgDB/WriteFile>

#include <osgViewer/Viewer>
#include <osgViewer/ViewerEventHandlers>
#include <osgViewer/CompositeViewer>
#include <osgGA/TrackballManipulator>
#include <osg/LineSegment>
#include <osgUtil/IntersectVisitor>

#include <osg/Camera>
#include <osg/RenderInfo>

#include <math.h>

// ACE includes
#include "ace/OS.h"
#include "ace/Log_Msg.h"
#include "ace/INET_Addr.h"
#include "ace/SOCK_Dgram_Mcast.h"

// CxImage includes
#include "ximage.h"
#include "ximagif.h"
#include "ximatif.h"
#include "xfile.h"

#include "TransformAccumulator.h"
#include "KeyboardHandler.h"
#include "orientation.h"
#include "FollowNodeManip.h"

#include "viewparams.h"

#include "xmlParser.h"
void ReadInXMLInfo();
//XML input terrain and vehicle model info
std::string sXMLTerrainFileName;
std::string sXMLTerrainPathName;

std::string sXMLVehicleFileName;
std::string sXMLVehiclePathName;

std::string sXMLMulticastAddress;
osgDB::FilePathList pathList;
double xOffsetToZero, yOffsetToZero, zOffsetToZero; // read in from XML file to position (0,0,0) on terrain

char tempstr[] = "                                        ";

// Global OSG constructs
osgViewer::View* view1  = new osgViewer::View();


osgViewer::CompositeViewer viewer;
osgGA::TrackballManipulator *Tman1 = new osgGA::TrackballManipulator();
osg::ref_ptr<osg::Camera> mainCamera = new osg::Camera;

extern osgViewer::View*  createViewNew (osgViewer::CompositeViewer *viewer,
                 osg::ref_ptr<osg::Group> scene,
                 osg::ref_ptr<osg::GraphicsContext> gc,
                 osgGA::TrackballManipulator* Tman,
                 int x, int y, int width, int height);

osg::Quat gViewquat; 
osg::Vec3 gTankPos;


int message_counter = 0;
float xViewPos = 0;
float yViewPos = -5.0;
float zViewPos = 5.0;
float viewAngPitch = 0.0f;
float viewAngYaw = 0.0f;
float viewAngRoll = 0.0f;

double tankXPosition = 100.0;
double tankYPosition = 0.0;
osg::PositionAttitudeTransform* tankXform;     // Transformation node
osg::Node* terrainNode = NULL;

int globalOSGHFOVValue = 60;
int globalOSGVFOVValue = 30;

// ACE multicast defines
#define MYBUFSIZE 32768
//#define DEFAULT_MULTICAST_ADDR "224.9.9.2"

// Terrain offsets to put UUT at desired location - no longer used in SceneGen app
//#define XOFFSET_MOUT0 -760.0
//#define YOFFSET_MOUT0 -1570.0
//#define ZOFFSET_MOUT0 -187.50



//TENA Variables
//RTTC::TA2PublisherImpl::SceneGenerator sceneGenerator;

// globals used to pass to MFC Text (or Edit Control) boxes
double gMFCfieldOfView;
double gMFCzoomFactor;
int    gMFCterrainID;
double gMFCviewFromX;
double gMFCviewFromY;
double gMFCviewFromZ;
double gMFCviewToX;
double gMFCviewToY;
double gMFCviewToZ;
double gMFCviewPsi;
double gMFCviewTheta;
double gMFCviewPhi;
//double gMFCviewRotationAngle;

bool   gNoPlace = false;  // flag to let us know if UUT can not terrain clamp (i.e. off terrain edge)
void SetOSGWindowViews(osgViewer::CompositeViewer viewer);
extern bool bResignOSG;

osg::PositionAttitudeTransform *followerPAT = new osg::PositionAttitudeTransform();
transformAccumulator* tankWorldCoords = new transformAccumulator();
osg::Node* tankNode = new osg::Node();

long frameCounter = 0;
#define FRAMESKIP 5

// Camera callback class
// Renders the current frame, converts it to a jepg image, and sends it out via
//  ACE udp multicast
// Developer : Mike Greene, Trideum Corp.
// Date Implemented : 04/09/2009
//

struct SnapImage : public osg::Camera::DrawCallback
{
    
    SnapImage()
    {
        _image = new osg::Image;        
    }

    virtual void operator () (osg::RenderInfo& renderInfo) const
    {

	  if (frameCounter%FRAMESKIP == 0) {
		// Set up multicast datagram
			ACE_INET_Addr multicast_addr_(2000,sXMLMulticastAddress.c_str());
			ACE_INET_Addr local_addr_((u_short)0);
			ACE_SOCK_Dgram dgram_(local_addr_);
    
        //osg::notify(osg::NOTICE)<<"Camera callback"<<std::endl;

        osg::Camera* camera = renderInfo.getCurrentCamera();
        osg::Viewport* viewport = camera ? camera->getViewport() : 0;

        //osg::notify(osg::NOTICE)<<"Camera callback "<<camera<<" "<<viewport<<std::endl;

        if (viewport && _image.valid())
        {
            _image->readPixels(int(viewport->x()),int(viewport->y()),int(viewport->width()),int(viewport->height()),
                               GL_RGB,
                               GL_UNSIGNED_BYTE);
           //osgDB::writeImageFile(*_image, _filename);
            
            //osg::notify(osg::NOTICE)<<"Taken screenshot, and written to '"<<_filename<<"'"<<std::endl;             
        

		// read back in jpeg from RAMDisk and reduce its quality
			CxImage  cimage;
			CxMemFile memfile;

			cimage.CreateFromArray(_image->data(),512,256,24,512*3,false);
			//cimage.Load("b:\\mg.jpg", CXIMAGE_FORMAT_JPG);
			cimage.SetJpegQuality(35);

			memfile.Open();
			cimage.Encode(&memfile,CXIMAGE_FORMAT_JPG);
			BYTE* buffer = memfile.GetBuffer();
			long size = memfile.Size();
			memfile.Close();

			// send out via udp multicast
			if(dgram_.send((void *) buffer,size,multicast_addr_)==-1) {
				MessageBox(NULL,"Error in MultiCast Send!","ERROR",MB_OK);
			}

			free(buffer);
			dgram_.close();
		}
	  }	
    }
    mutable osg::ref_ptr<osg::Image>    _image;
};

// Get needed TENA values to calculate correct look from/to value for rendering frame to be 
//   sent out
// Developer : Mike Greene, Trideum Corp.
// Date Implemented : 10/08/2006
//
void UpdateTankTSPI(void)
{
   // Get TENA vars
/*	RTTC::TA2PublisherImpl::TASemulator TAS_ = SCENE_GENERATOR_PUBLISHER->GetTAS();
   // set externs to use in MFC display variables
	gMFCfieldOfView			= TAS_.fieldOfView;
	gMFCzoomFactor			= TAS_.zoomFactor;
	gMFCterrainID			= TAS_.terrainID;
	gMFCviewFromX			= TAS_.viewFromX;
	gMFCviewFromY			= TAS_.viewFromY;
	gMFCviewFromZ			= TAS_.viewFromZ;
	gMFCviewToX				= TAS_.viewToX;
	gMFCviewToY				= TAS_.viewToY;
	//gMFCviewToZ				= TAS_.viewFromZ-0.25;   //TAS_.viewToZ;
	gMFCviewRotationAngle	= TAS_.viewRotationAngle;*/

	// mg test
    osg::Vec3 mgtankPos;
	mgtankPos.set(gMFCviewFromX,gMFCviewFromY,gMFCviewFromZ);
	tankXform->setPosition(mgtankPos);
    osg::Quat mgquat;
	// note - because of TENA OM errors, these values do not correspond to correctly ID's vars on OM
	Orientation::Euler_to_Quaternian(gMFCviewPsi,gMFCviewTheta,gMFCviewPhi,
		mgquat.w(),mgquat.x(),mgquat.y(),mgquat.z());
	//tankXform->setAttitude(mgquat);
	//return;

	// end mgtest


	// use line segment to project into terrain to determine intersection point
		osg::LineSegment* tankLocationSegment = new osg::LineSegment();
		tankLocationSegment->set(
			osg::Vec3(gMFCviewFromX, gMFCviewFromY, 9999) ,
			osg::Vec3(gMFCviewFromX, gMFCviewFromY, -9999) );

		osgUtil::IntersectVisitor findTankElevationVisitor;
		findTankElevationVisitor.addLineSegment(tankLocationSegment);
		terrainNode->accept(findTankElevationVisitor);

		osgUtil::IntersectVisitor::HitList tankElevationLocatorHits;
	// if intersection, save hit list (might be more than 1)
		tankElevationLocatorHits = findTankElevationVisitor.getHitList(tankLocationSegment);
		osgUtil::Hit heightTestResults;
		
		// if no hits, uut must not be on terrain
		if ( tankElevationLocatorHits.empty() )
		{
			gNoPlace = true;
			//return -1;
		}
		else {
			heightTestResults = tankElevationLocatorHits.front();
			osg::Vec3 terrainHeight = heightTestResults.getWorldIntersectPoint();
			osg::Vec3d np = heightTestResults.getLocalIntersectNormal();
      // update tank position with new up location
			
			gTankPos.set(terrainHeight.x(),terrainHeight.y(),terrainHeight.z());

			// constant up vector 
			osg::Vec3d up ( 0.0f, 0.0f, 1.0f ); 
			//osg::Quat quat; 
			// make the rotation from the up vector to the normal 
			gViewquat.makeRotate ( up, np ); 
			// mg retrieve the terrain following Eulers
			double mpsi, mtheta, mphi;
			Orientation::Quaternian_to_Euler(gViewquat.w(),gViewquat.x(),gViewquat.y(),gViewquat.z(),
					mpsi,mtheta,mphi);
			mpsi   += gMFCviewPsi;
			mtheta += gMFCviewTheta;
			mphi   += gMFCviewPhi;

			Orientation::Euler_to_Quaternian(mpsi,mtheta,mphi,
				gViewquat.w(),gViewquat.x(),gViewquat.y(),gViewquat.z());
	 // update tank's orientation based on calculated pitch and roll
			tankXform->setAttitude(gViewquat);
		}
}

// Main OSG loop routine
// Developer : Mike Greene, Trideum Corp.
// Date Implemented : 04/09/2009
//
void OSG_Module(void)
{
   osg::Group* rootNode = new osg::Group();

   // read in path and model initialization from xml file
   pathList = osgDB::getDataFilePathList();
   ReadInXMLInfo();
  
   osgDB::setDataFilePathList(pathList);

   // setup Light source to act as Sun
   osg::Light * light = new osg::Light; 
   light->setLightNum (0); 

   // Set Ambient, Diffuse, And Specular values
    float sunAmbient = 0.5f; 
    float sunDiffuse = 0.6f; 
    float sunSpecular = 0.2f; 

    light->setDiffuse (osg::Vec4 (sunDiffuse, sunDiffuse, sunDiffuse, 1.0f)); 
    light->setAmbient (osg::Vec4 (sunAmbient, sunAmbient, sunAmbient, 1.0f)); 
    light->setSpecular (osg::Vec4 (sunSpecular, sunSpecular, sunSpecular, 1.0f)); 

    // Set Sun Position and direction
    light->setPosition (osg::Vec4(0.0f, 0.0f, 1000.0f, 0.0f)); 
    light->setDirection (osg::Vec3 (0.0f, 0.0f, 0.0f)); 

    // Add to scene tree
    osg::LightSource* sun = new osg::LightSource; 
    sun->setLight (light); 
    rootNode->addChild (sun); 

	osg::PositionAttitudeTransform* terrainPAT = new osg::PositionAttitudeTransform();

   tankNode = osgDB::readNodeFile(sXMLVehicleFileName);
  
   terrainNode = osgDB::readNodeFile(sXMLTerrainFileName); 

   if (! (tankNode && terrainNode))
   {
      MessageBox(NULL,"Couldn't load models, quitting.","Error!",MB_OK);
      //return -1;
   }

   // set up cameras
   osg::GraphicsContext::WindowingSystemInterface* wsi = osg::GraphicsContext::getWindowingSystemInterface();
	unsigned int width, height;
    wsi->getScreenResolution(osg::GraphicsContext::ScreenIdentifier(0), width, height);
	
	osg::ref_ptr<osg::GraphicsContext::Traits> traits = new osg::GraphicsContext::Traits;
	traits->screenNum = 0;
    traits->x = 500;
    traits->y = 30;
    traits->width = 512;
    traits->height = 256;
    traits->windowDecoration = true;
    traits->doubleBuffer = true;
    traits->sharedContext = 0;

    osg::ref_ptr<osg::GraphicsContext> gc = osg::GraphicsContext::createGraphicsContext(traits.get());
    if (gc.valid())
    {
        osg::notify(osg::INFO)<<"  GraphicsWindow has been created successfully."<<std::endl;

        // need to ensure that the window is cleared make sure that the complete window is set the correct colour
        // rather than just the parts of the window that are under the camera's viewports
        gc->setClearColor(osg::Vec4f(0.2f,0.2f,0.6f,1.0f));
        gc->setClearMask(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    }
    else
    {
        osg::notify(osg::NOTICE)<<"  GraphicsWindow has not been created successfully."<<std::endl;
    }

	mainCamera->setGraphicsContext(gc.get());
	mainCamera->setViewport(new osg::Viewport(0,0, 512,256));
	
	GLenum buffer = traits->doubleBuffer ? GL_BACK : GL_FRONT;
    mainCamera->setDrawBuffer(buffer);
    mainCamera->setReadBuffer(buffer);

	Tman1->setDataVariance(osg::Object::DYNAMIC);
	view1 = createViewNew (&viewer,rootNode,gc,Tman1,   0,  0, traits->width,traits->height); //normal view
	
	view1->setDataVariance(osg::Object::DYNAMIC);
	viewer.setDataVariance(osg::Object::DYNAMIC);

    view1->setSceneData( rootNode );
  

   // set transform to place camera correctly relative to incoming TSPI position
   osg::PositionAttitudeTransform * followerOffset =
      new osg::PositionAttitudeTransform();
   followerOffset->setPosition( osg::Vec3(0.0,2.0,5.0) );//( osg::Vec3(0.0,-25.0,10) );
   followerOffset->setAttitude(
      osg::Quat( osg::DegreesToRadians(-2.0), osg::Vec3(1,0,0) ,
	             osg::DegreesToRadians(0.0), osg::Vec3(0,1,0) ,
				 osg::DegreesToRadians(0.0), osg::Vec3(0,0,1) ));  //15.0

   // setup scene graph
   rootNode->addChild(terrainPAT);
   terrainPAT->addChild(terrainNode);

   tankXform = new osg::PositionAttitudeTransform();

   rootNode->addChild(tankXform);
   tankXform->addChild(followerOffset);
   tankXform->addChild(tankNode);

   tankWorldCoords->attachToGroup(followerOffset);

   // create the windows and run the threads.
   viewer.realize();

   osg::Vec4 clearColor(0.0, 0.0, 0.0, 1.0);
   view1->getCamera()->setClearColor(clearColor);

   // not really performing a function now as offset is included in incoming TSPI
   osg::Vec3f terrainAdjustmentVec(0.0,0.0,0.0);
   terrainPAT->setPosition(terrainAdjustmentVec);

   //make tank invisible
   tankNode->setNodeMask(0);

   SnapImage* postDrawCallback = new SnapImage();
   view1->getCamera()->setPostDrawCallback(postDrawCallback);  

   while( !viewer.done() )
   {  
	  // Get new TENA tspi values each loop
	  UpdateTankTSPI();

	  // based on user input, set up views
	  SetOSGWindowViews(viewer);

	  if (bResignOSG) viewer.setDone(true);

	  frameCounter++;

      // fire off the cull and draw traversals of the scene.
      viewer.frame();
   }
}

// Routine to read in OSG terrain and UUT model and path information
// Mike Greene  -  Trideum Corp.   July 17,2007
//  (256)  704-6113
void ReadInXMLInfo()
{
	 // this opens and parses the XML file:
	XMLNode xMainNode=XMLNode::openFileHelper("SceneGeneratorconfig.xml","MGXML");

    XMLNode xNode=xMainNode.getChildNode("terrainfile");
	sXMLTerrainFileName = std::string(xNode.getText());
	//MessageBox(NULL,xNode.getText(),"TerrainFileName",MB_OK);

	xNode=xMainNode.getChildNode("vehiclefile");
	sXMLVehicleFileName = std::string(xNode.getText());
	//MessageBox(NULL,xNode.getText(),"VehicleFileName",MB_OK);

	xNode=xMainNode.getChildNode("xoffset");
	xOffsetToZero = atof(xNode.getText());
	//MessageBox(NULL,xNode.getText(),"VehicleFileName",MB_OK);

	xNode=xMainNode.getChildNode("yoffset");
	yOffsetToZero = atof(xNode.getText());
	//MessageBox(NULL,xNode.getText(),"VehicleFileName",MB_OK);

	xNode=xMainNode.getChildNode("zoffset");
	zOffsetToZero = atof(xNode.getText());
	//MessageBox(NULL,xNode.getText(),"VehicleFileName",MB_OK);

	xNode=xMainNode.getChildNode("multicast_address");
	sXMLMulticastAddress = std::string(xNode.getText());
	//MessageBox(NULL,xNode.getText(),"VehicleFileName",MB_OK);

	// find number of "modelpath" tags
	int np=xMainNode.nChildNode("modelpath");
	//sprintf(ts,"%d = numpaths",np);
	//MessageBox(NULL,ts,"",MB_OK);
	

	int j, polyIterator=0;
	for (j=0 ; j < np; j++)
	{
		// this gets the number data from the "path" tags and pushes them onto the search list:
		
		xNode = xMainNode.getChildNode("modelpath",&polyIterator).getChildNode("path");
		//MessageBox(NULL,xNode.getText(),"Path = ",MB_OK);
		pathList.push_back(xNode.getText());	
	}
    
}

// Routine to set up user defined views for top and bottom windows
// Mike Greene - Trideum Corp.   July 17, 2007
//  (256) 704-6113
void SetOSGWindowViews(osgViewer::CompositeViewer viewer)
{
	// local vars used to manipulate slider values
   double rotateAngle = 0.0;
   double panAng  = 0.0;
   double tiltAng = 0.0;
//
   osg::Quat registerQuat;
   followNodeMatrixManipulator* followTank = new followNodeMatrixManipulator(tankWorldCoords);

			tankNode->setNodeMask(0);
			followerPAT->setPosition(osg::Vec3d(video_register_x, video_register_y+23.0, video_register_z-2.5));
            Orientation::Euler_to_Quaternian(osg::DegreesToRadians(video_register_pan),
	                               osg::DegreesToRadians(video_register_roll),
								   osg::DegreesToRadians(video_register_tilt),
		                           registerQuat.w(),registerQuat.x(),registerQuat.y(),registerQuat.z());
			followerPAT->setAttitude(registerQuat);
           
			view1->setCameraManipulator(followTank);
			

}